package java.security;

import java.security.spec.InvalidKeySpecException;
import java.security.spec.KeySpec;
import sun.security.jca.GetInstance;
import java.util.Iterator;
import sun.security.util.Debug;

public class KeyFactory
{
    private static final Debug debug;
    private final String algorithm;
    private Provider provider;
    private volatile KeyFactorySpi spi;
    private final Object lock;
    private Iterator<Provider.Service> serviceIterator;
    
    protected KeyFactory(KeyFactorySpi spi, Provider provider, String algorithm) {
        this.lock = new Object();
        this.spi = spi;
        this.provider = provider;
        this.algorithm = algorithm;
    }
    
    private KeyFactory(String str) throws NoSuchAlgorithmException {
        this.lock = new Object();
        this.algorithm = str;
        this.serviceIterator = GetInstance.getServices("KeyFactory", str).iterator();
        if (this.nextSpi(null) == null) {
            throw new NoSuchAlgorithmException(str + " KeyFactory not available");
        }
        return;
    }
    
    public static KeyFactory getInstance(String s) throws NoSuchAlgorithmException {
        return new KeyFactory(s);
    }
    
    public static KeyFactory getInstance(String algorithm, String provider) throws NoSuchAlgorithmException, NoSuchProviderException {
        GetInstance.Instance instance = GetInstance.getInstance("KeyFactory", KeyFactorySpi.class, algorithm, provider);
        return new KeyFactory((KeyFactorySpi)instance.impl, instance.provider, algorithm);
    }
    
    public static KeyFactory getInstance(String algorithm, Provider provider) throws NoSuchAlgorithmException {
        GetInstance.Instance instance = GetInstance.getInstance("KeyFactory", KeyFactorySpi.class, algorithm, provider);
        return new KeyFactory((KeyFactorySpi)instance.impl, instance.provider, algorithm);
    }
    
    public final Provider getProvider() {
        synchronized (this.lock) {
            this.serviceIterator = null;
            return this.provider;
        }
    }
    
    public final String getAlgorithm() {
        return this.algorithm;
    }
    
    private KeyFactorySpi nextSpi(KeyFactorySpi keyFactorySpi) {
        synchronized (this.lock) {
            if (keyFactorySpi != null && keyFactorySpi != this.spi) {
                return this.spi;
            }
            if (this.serviceIterator == null) {
                return null;
            }
            while (this.serviceIterator.hasNext()) {
                Provider.Service service = (Provider.Service)this.serviceIterator.next();
                try {
                    Object instance = service.newInstance(null);
                    if (!(instance instanceof KeyFactorySpi)) {
                        continue;
                    }
                    KeyFactorySpi spi = (KeyFactorySpi)instance;
                    this.provider = service.getProvider();
                    this.spi = spi;
                    return spi;
                }
                catch (NoSuchAlgorithmException ex) {}
            }
            this.serviceIterator = null;
            return null;
        }
    }
    
    public final PublicKey generatePublic(KeySpec keySpec) throws InvalidKeySpecException {
        if (this.serviceIterator == null) {
            return this.spi.engineGeneratePublic(keySpec);
        }
        Exception cause = null;
        KeyFactorySpi keyFactorySpi = this.spi;
        try {
            return keyFactorySpi.engineGeneratePublic(keySpec);
        }
        catch (Exception ex) {
            if (cause == null) {
                cause = ex;
            }
            keyFactorySpi = this.nextSpi(keyFactorySpi);
            if (keyFactorySpi != null) {
                return keyFactorySpi.engineGeneratePublic(keySpec);
            }
            if ((cause instanceof RuntimeException)) {
                throw (RuntimeException)cause;
            }
            if ((cause instanceof InvalidKeySpecException)) {
                throw (InvalidKeySpecException)cause;
            }
            throw new InvalidKeySpecException("Could not generate public key", cause);
        }
    }
    
    public final PrivateKey generatePrivate(KeySpec keySpec) throws InvalidKeySpecException {
        if (this.serviceIterator == null) {
            return this.spi.engineGeneratePrivate(keySpec);
        }
        Exception cause = null;
        KeyFactorySpi keyFactorySpi = this.spi;
        try {
            return keyFactorySpi.engineGeneratePrivate(keySpec);
        }
        catch (Exception ex) {
            if (cause == null) {
                cause = ex;
            }
            keyFactorySpi = this.nextSpi(keyFactorySpi);
            if (keyFactorySpi != null) {
                return keyFactorySpi.engineGeneratePrivate(keySpec);
            }
            if ((cause instanceof RuntimeException)) {
                throw (RuntimeException)cause;
            }
            if ((cause instanceof InvalidKeySpecException)) {
                throw (InvalidKeySpecException)cause;
            }
            throw new InvalidKeySpecException("Could not generate private key", cause);
        }
    }
    
    public final <T extends KeySpec> T getKeySpec(Key key, Class<T> clazz) throws InvalidKeySpecException {
        if (this.serviceIterator == null) {
            return this.spi.engineGetKeySpec(key, clazz);
        }
        Exception cause = null;
        KeyFactorySpi keyFactorySpi = this.spi;
        try {
            return keyFactorySpi.engineGetKeySpec(key, clazz);
        }
        catch (Exception ex) {
            if (cause == null) {
                cause = ex;
            }
            keyFactorySpi = this.nextSpi(keyFactorySpi);
            if (keyFactorySpi != null) {
                return keyFactorySpi.engineGetKeySpec(key, clazz);
            }
            if ((cause instanceof RuntimeException)) {
                throw (RuntimeException)cause;
            }
            if ((cause instanceof InvalidKeySpecException)) {
                throw (InvalidKeySpecException)cause;
            }
            throw new InvalidKeySpecException("Could not get key spec", cause);
        }
    }
    
    public final Key translateKey(Key key) throws InvalidKeyException {
        if (this.serviceIterator == null) {
            return this.spi.engineTranslateKey(key);
        }
        Exception ex = null;
        KeyFactorySpi keyFactorySpi = this.spi;
        try {
            return keyFactorySpi.engineTranslateKey(key);
        }
        catch (Exception ex2) {
            if (ex == null) {
                ex = ex2;
            }
            keyFactorySpi = this.nextSpi(keyFactorySpi);
            if (keyFactorySpi != null) {
                return keyFactorySpi.engineTranslateKey(key);
            }
            if ((ex instanceof RuntimeException)) {
                throw (RuntimeException)ex;
            }
            if ((ex instanceof InvalidKeyException)) {
                throw (InvalidKeyException)ex;
            }
            throw new InvalidKeyException("Could not translate key", ex);
        }
    }
    
    static {
        debug = Debug.getInstance("jca", "KeyFactory");
    }
}
